@@ -8,8 +8,11 @@ use crate::{
 use glib::Sender;
 use std::path::PathBuf;
 
-use gtk::{AccelGroup, WidgetExt};
+use gtk::{prelude::WidgetExt, AccelGroup};
+#[cfg(not(feature = "ayatana"))]
 use libappindicator::{AppIndicator, AppIndicatorStatus};
+#[cfg(feature = "ayatana")]
+use libayatana_appindicator::{AppIndicator, AppIndicatorStatus};
 
 use super::{menu::Menu, window::WindowRequest, WindowId};
 
