@@ -1,9 +1,10 @@
 // Copyright 2019-2021 Tauri Programme within The Commons Conservancy
 // SPDX-License-Identifier: Apache-2.0
 
-use raw_window_handle::{macos::MacOSHandle, RawWindowHandle};
+use raw_window_handle::{AppKitHandle, RawWindowHandle};
 use std::{
   collections::VecDeque,
+  convert::TryInto,
   f64,
   os::raw::c_void,
   sync::{
@@ -38,7 +39,7 @@ use cocoa::{
     NSWindowStyleMask,
   },
   base::{id, nil},
-  foundation::{NSAutoreleasePool, NSDictionary, NSPoint, NSRect, NSSize},
+  foundation::{NSAutoreleasePool, NSDictionary, NSPoint, NSRect, NSSize, NSUInteger},
 };
 use core_graphics::display::{CGDisplay, CGDisplayMode};
 use objc::{
@@ -295,7 +296,10 @@ pub struct SharedState {
   is_simple_fullscreen: bool,
   pub saved_style: Option<NSWindowStyleMask>,
   /// Presentation options saved before entering `set_simple_fullscreen`, and
-  /// restored upon exiting it
+  /// restored upon exiting it. Also used when transitioning from Borderless to
+  /// Exclusive fullscreen in `set_fullscreen` because we need to disable the menu
+  /// bar in exclusive fullscreen but want to restore the original options when
+  /// transitioning back to borderless fullscreen.
   save_presentation_opts: Option<NSApplicationPresentationOptions>,
   pub saved_desktop_display_mode: Option<(CGDisplay, CGDisplayMode)>,
 }
@@ -344,7 +348,8 @@ impl UnownedWindow {
     pl_attribs: PlatformSpecificWindowBuilderAttributes,
   ) -> Result<(Arc<Self>, IdRef), RootOsError> {
     unsafe {
-      if !msg_send![class!(NSThread), isMainThread] {
+      let is_main_thread: BOOL = msg_send!(class!(NSThread), isMainThread);
+      if is_main_thread == NO {
         panic!("Windows can only be created on the main thread on macOS");
       }
     }
@@ -843,6 +848,15 @@ impl UnownedWindow {
 
       let mut fade_token = ffi::kCGDisplayFadeReservationInvalidToken;
 
+      if matches!(old_fullscreen, Some(Fullscreen::Borderless(_))) {
+        unsafe {
+          let app = NSApp();
+          trace!("Locked shared state in `set_fullscreen`");
+          let mut shared_state_lock = self.shared_state.lock().unwrap();
+          shared_state_lock.save_presentation_opts = Some(app.presentationOptions_());
+        }
+      }
+
       unsafe {
         // Fade to black (and wait for the fade to complete) to hide the
         // flicker from capturing the display and switching display mode
@@ -932,16 +946,41 @@ impl UnownedWindow {
         // of the menu bar, and this looks broken, so we must make sure
         // that the menu bar is disabled. This is done in the window
         // delegate in `window:willUseFullScreenPresentationOptions:`.
+        let app = NSApp();
+        trace!("Locked shared state in `set_fullscreen`");
+        shared_state_lock.save_presentation_opts = Some(app.presentationOptions_());
+
+        let presentation_options =
+          NSApplicationPresentationOptions::NSApplicationPresentationFullScreen
+            | NSApplicationPresentationOptions::NSApplicationPresentationHideDock
+            | NSApplicationPresentationOptions::NSApplicationPresentationHideMenuBar;
+        app.setPresentationOptions_(presentation_options);
+
         let () = msg_send![*self.ns_window, setLevel: ffi::CGShieldingWindowLevel() + 1];
       },
       (
         &Some(Fullscreen::Exclusive(RootVideoMode { ref video_mode })),
         &Some(Fullscreen::Borderless(_)),
       ) => unsafe {
+        let presentation_options = shared_state_lock.save_presentation_opts.unwrap_or_else(|| {
+          NSApplicationPresentationOptions::NSApplicationPresentationFullScreen
+            | NSApplicationPresentationOptions::NSApplicationPresentationAutoHideDock
+            | NSApplicationPresentationOptions::NSApplicationPresentationAutoHideMenuBar
+        });
+        NSApp().setPresentationOptions_(presentation_options);
+
         util::restore_display_mode_async(video_mode.monitor().inner.native_identifier());
+
+        // Restore the normal window level following the Borderless fullscreen
+        // `CGShieldingWindowLevel() + 1` hack.
+        let () = msg_send![
+          *self.ns_window,
+          setLevel: ffi::NSWindowLevel::NSNormalWindowLevel
+        ];
       },
       _ => INTERRUPT_EVENT_LOOP_EXIT.store(false, Ordering::SeqCst),
     }
+    trace!("Unlocked shared state in `set_fullscreen`");
   }
 
   #[inline]
@@ -1052,9 +1091,9 @@ impl UnownedWindow {
       let desc = NSScreen::deviceDescription(screen);
       let key = util::ns_string_id_ref("NSScreenNumber");
       let value = NSDictionary::valueForKey_(desc, *key);
-      let display_id = msg_send![value, unsignedIntegerValue];
+      let display_id: NSUInteger = msg_send![value, unsignedIntegerValue];
       RootMonitorHandle {
-        inner: MonitorHandle::new(display_id),
+        inner: MonitorHandle::new(display_id.try_into().unwrap()),
       }
     }
   }
@@ -1077,12 +1116,10 @@ impl UnownedWindow {
 
   #[inline]
   pub fn raw_window_handle(&self) -> RawWindowHandle {
-    let handle = MacOSHandle {
-      ns_window: *self.ns_window as *mut _,
-      ns_view: *self.ns_view as *mut _,
-      ..MacOSHandle::empty()
-    };
-    RawWindowHandle::MacOS(handle)
+    let mut handle = AppKitHandle::empty();
+    handle.ns_window = *self.ns_window as *mut _;
+    handle.ns_view = *self.ns_view as *mut _;
+    RawWindowHandle::AppKit(handle)
   }
 }
 
